//
// implementation of vector spaces and other simple routines used by both programs
//.................................................................................

using namespace std;
#include <vector>
#include <stdio.h>
#include <math.h>

// abbreviations for names of used types
typedef long double ld;
typedef long long ll;

// useful macros
#define REP(x,n) for(int x=0; x<(n); x++) // for x = 0 to n-1; declare x
#define REPD(x,n) for(x=0; x<(n); x++)    // for x = 0 to n-1;
#define SIGN(x) for(x=-1; x<2; x+=2)      // for x in {-1, +1}
#define SIGZ(x) for(x=-1; x<2; x++)       // for x in {-1, 0, +1}
#define SIZE(x) (int((x).size()))         // size of a vector

// the golden proportion
ld phi = (1+sqrt(5))/2;

// is a in range [b,c]?
bool ir(int a, int b, int c) { return a>=b && a<=c; }

// a vector with coordinates of type T and dim dimensions
template<class T, int dim> struct vec {
  T x[dim];
  T& operator [] (int i) { return x[i]; }
  };

// we use the following instances:
typedef vec<ld,  4> vec4;
typedef vec<ld,  3> vec3;
typedef vec<ld,  2> vec2;
typedef vec<int, 3> ivec;
typedef vec<ll,  3> lvec;

// basic vector operations

template<class T, int dim> bool operator == (vec<T,dim>& a, vec<T,dim> b) {
  for(int i=0; i<dim; i++) if(a[i] != b[i]) return false;
  return true;
  }

template<class T, int dim> vec<T,dim>& operator += (vec<T,dim>& a, vec<T,dim> b) {
  for(int i=0; i<dim; i++) a[i] += b[i]; return a;
  }

template<class T, int dim> vec<T,dim>& operator -= (vec<T,dim>& a, vec<T,dim> b) {
  for(int i=0; i<dim; i++) a[i] -= b[i]; return a;
  }

template<class T, int dim> vec<T,dim>& operator *= (vec<T,dim>& a, ld k) {
  for(int i=0; i<dim; i++) a[i] *= k; return a;
  }

template<class T, int dim> vec<T,dim>& operator /= (vec<T,dim>& a, ld k) {
  for(int i=0; i<dim; i++) a[i] /= k; return a;
  }

template<class T, int dim> vec<T,dim> operator + (vec<T,dim> a, vec<T,dim> b) {
  a += b; return a;
  }

template<class T, int dim> vec<T,dim> operator - (vec<T,dim> a, vec<T,dim> b) {
  a -= b; return a;
  }

template<class T, int dim> vec<T,dim> operator * (ld x, vec<T,dim> a) {
  a *= x; return a;
  }

template<class T, int dim> vec<T,dim> operator * (vec<T,dim> a, ld x) {
  a *= x; return a;
  }

template<class T, int dim> vec<T,dim> operator / (vec<T,dim> a, ld x) {
  a /= x; return a;
  }

// inner product
template<class T, int dim> T operator | (vec<T,dim> a, vec<T,dim> b) {
  T ret = 0; int i;
  REP(i,dim) ret += a[i] * b[i];
  return ret;
  }

// cross product
template <class T> vec<T,3> operator ^ (vec<T,3> a, vec<T,3> b) {
  vec<T,3> c;
  c[0] = a[1] * b[2] - a[2] * b[1];
  c[1] = a[2] * b[0] - a[0] * b[2];
  c[2] = a[0] * b[1] - a[1] * b[0];
  return c;
  }

// squared vector's length
template<class T, int dim> T norm(vec<T,dim> a) { return (a|a); }

// vector's length
template<class T, int dim> ld abs (vec<T,dim> a) { return sqrt(norm(a)); }

// rotate a 2d vector by an angle given by S (sin) and C (cos)
void roll(ld& x, ld& y, ld S, ld C) {

  ld xn = x*C - y*S;
  ld yn = x*S + y*C;
  
  x = xn; y = yn;
  }

// convert ivec to lvec
lvec asLL(ivec i) {
  lvec v; v[0] = i[0]; v[1] = i[1]; v[2] = i[2];
  return v;
  }

// absolute value of a ll
ll loabs(ll x) { return x>0?x:-x; }

// volume of a tetrahedron with given vertices
// (actually returns 6x the volume)

ll volume(lvec a, lvec b, lvec c, lvec d) {
  b -= a; c -= a; d -= a;
  
  return loabs((b^c) | d);
  }

ld volume(vec3 a, vec3 b, vec3 c, vec3 d) {
  b -= a; c -= a; d -= a;
  
  return fabs((b^c) | d);
  }

// a stereographic projection of a vec4
vec3 project(vec4 p) {
  vec3 v; int i;
  REP(i,3) v[i] = p[i] / (1-p[3]);
  return v;
  }

// the plane going through the points a,b,c is the set of points 
// x such that (x|v) is 1, where v = planevia(a,b,c)

vec3 planevia(vec3 a, vec3 b, vec3 c) {
  
  // we rotate the scene by a 'random' angle so that we don't have
  // to implement the special case of dividing by 0 
  ld S = sin(.1), C = cos(.1);
  
  roll(a[0], a[1], S, C);
  roll(b[0], b[1], S, C);
  roll(c[0], c[1], S, C);
  
  ld ab01 = a[0]*b[1] - b[0]*a[1];
  ld ab02 = a[0]*b[2] - b[0]*a[2];

  ld ac01 = a[0]*c[1] - c[0]*a[1];
  ld ac02 = a[0]*c[2] - c[0]*a[2];
  
  ld ab0 = a[0]-b[0]; ld ac0 = a[0]-c[0];
  
  vec3 v;
  
  v[2] = (ab0*ac01-ac0*ab01) / (ab02*ac01-ab01*ac02);
  v[1] = (ab0*ac02-ac0*ab02) / (ab01*ac02-ab02*ac01);
  v[0] = (1 - a[2]*v[2] - a[1]*v[1]) / a[0];
  
  roll(v[0], v[1], -S, C); // unroll
  
  return v;
  }

// zero vectors

vec3 zero3;
vec4 zero4;
